@@ -18,6 +18,7 @@ import ConfigParser
 import os
 import re
 import sys
+import shutil
 from glob import glob
 from tempfile import mkdtemp
 
@@ -25,11 +26,17 @@ from iw.releaser import base
 
 join = os.path.join
 
-extract_path = re.compile(r'^(htt(?:p|ps)://)(.*?)/?$')
+extract_path = re.compile(r'^((?:htt(?:p|ps)|svn)://)(.*?)/?$')
 
 def _log(msg):
     print msg
 
+def _extract_url(url):
+    extracted = extract_path.findall(url)
+    protocol = extracted[0][0]
+    path = extracted[0][1].split('/')
+    return protocol, path
+
 def make_release(version=None):
     """this is called when a release is made over a buildout"""
 
@@ -47,9 +54,7 @@ def make_release(version=None):
     url = base.get_svn_url()
 
     # let's get the releases base folder
-    extracted = extract_path.findall(url)
-    protocol = extracted[0][0]
-    path = extracted[0][1].split('/')
+    protocol, path = _extract_url(url)
     path[-1] = 'releases'
     releases = '%s%s' % (protocol, '/'.join(path))
 
@@ -65,6 +70,20 @@ def make_release(version=None):
     # now let's create the branch
     base.svn_copy(url, release, 'creating %s release for project' % version)
 
+    # let's get it to add a version.txt file in the buildout root
+    rep = mkdtemp()
+    try:
+        base.svn_checkout(release, rep)  
+        os.chdir(rep)
+        version_file = os.path.join(rep, 'version.txt')
+        open(version_file, 'w').write(version)
+        base.svn_add(version_file)
+        msg = 'Added version file to buildout.'
+        _log(msg)
+        base.svn_commit(msg)
+    finally:
+        shutil.rmtree(rep)
+
 def parse_url(url):
     """return base_url, cfg::
 
@@ -116,7 +135,10 @@ def diff_releases(old=None, new=None, result=None):
 
     if result is None:
         result = '%s-to-%s.tgz' % (root_old_name, root_new_name)
-
+        working_dir = os.path.realpath(os.curdir)
+    else:
+        working_dir, result = os.path.split(result)
+        working_dir = os.path.realpath(working_dir)
     
     old_files = {}
     for f in old_tarball.getmembers():
@@ -140,9 +162,12 @@ def diff_releases(old=None, new=None, result=None):
     # now writing the diff tarball
     _log('Writing archive')
     archive_contents(result, tmp)
+    # the tarball is create in the folder up `tmp`, let's move it
+    topdir = os.path.split(tmp)[0]
+    os.rename(join(topdir, result), join(working_dir, result))
     _log('Diff done.')
 
-def archive_contents(archive, location, previous=None):
+def archive_contents(archive, location, exclude=None):
     """generates the tarball"""
     location = os.path.realpath(location)
     # we want a relative storage
@@ -150,11 +175,23 @@ def archive_contents(archive, location, previous=None):
     os.chdir(location)
     dirname, name = os.path.split(location)
     tar = base.TarFile.open(join(dirname, archive), 'w:gz')
+    if exclude is None:
+        exclude = []
+    else:
+        exclude = [os.path.realpath(os.path.join(location, sub))
+                   for sub in exclude]
     try:
         for root, dirs, filenames in os.walk('.'):
             if '.svn' in root.split(os.path.sep):
                 continue
-            # archiving empty dirs
+            skip = False
+            for excluded in exclude:
+                if os.path.realpath(root).startswith(excluded):
+                    skip = True
+                    break
+            if skip:
+                continue
+            # archiving empty dirs too
             for dir_ in dirs:   
                 if '.svn' in dir_.split(os.path.sep):
                     continue
@@ -202,11 +239,29 @@ def _python():
                 return found
     return 'python'
 
+def _set_dynlibs(root):
+    """win32: Makes sure libpython*.a is copied beside the Python executable"""
+    main_dir = os.path.dirname(sys.executable)
+    lib_dir = join(main_dir, 'libs')
+    name = 'libpython*.a'
+    libs = glob(join(lib_dir, name))
+    libs_dir = join(root, 'libs')
+    if not os.path.exists(libs_dir):
+        os.mkdir(libs_dir)
+    for lib in libs:
+        libfilename = os.path.split(lib)[-1]
+        shutil.copy(lib, join(libs_dir, libfilename))
+
 def deploy_release(path=None, target=None, archiving='full'):
     """deploy a release in-place"""
     print 'Checking python version ...'
     print '%s ok.' % check_python()
 
+    # XXX at this time we are rebuilding everything
+    # a pre-built release will have these subdirectories
+    # included to avoid compiling again
+    exclude = ['parts', 'var', 'develop-eggs', 'bin', 'lib']
+
     if path is None:
         if len(sys.argv) < 2:
             print 'usage : project_deploy [http://to/your/buildout/]config.cfg'
@@ -247,7 +302,11 @@ def deploy_release(path=None, target=None, archiving='full'):
         main()
     finally:
         sys.argv = old
-
+        
+    # create the libs folder if needed
+    if sys.platform == 'win32':
+        _set_dynlibs(root)
+           
     # now we can run bootstrap.py
     python = _python()
 
@@ -295,7 +354,7 @@ def deploy_release(path=None, target=None, archiving='full'):
         if archiving == 'none':
             return
         archive = 'release-%s.tgz' % release_name
-        archive_contents(archive, '.')
+        archive_contents(archive, '.', exclude)
         print '%s ok.' % archive
     finally:
         # setting back the original buildout.cfg file    
